@@ -10,10 +10,10 @@ import dataclasses
 from pathlib import Path
 
 from . import Utils
-from .FileSectionType import FileSectionType
-from .ContextSymbols import ContextRelocInfo
+from .ContextSymbols import ContextSymbol
 from .SymbolsSegment import SymbolsSegment
 from .GlobalOffsetTable import GlobalOffsetTable
+from .Relocation import RelocationInfo, RelocType
 
 
 @dataclasses.dataclass
@@ -52,12 +52,8 @@ class Context:
         self.bannedSymbols: set[int] = set()
         self.bannedRangedSymbols: list[SymbolRange] = list()
 
-        self.relocInfosPerSection: dict[FileSectionType, dict[int, ContextRelocInfo]] = {
-            FileSectionType.Text: dict(),
-            FileSectionType.Data: dict(),
-            FileSectionType.Rodata: dict(),
-            FileSectionType.Bss: dict(),
-        }
+        self.globalRelocationOverrides: dict[int, RelocationInfo] = dict()
+        "key: vrom address"
 
         self.got: GlobalOffsetTable = GlobalOffsetTable()
 
@@ -91,16 +87,6 @@ class Context:
         return segment
 
 
-    def getRelocInfo(self, vram: int, sectionType: FileSectionType) -> ContextRelocInfo|None:
-        relocsInSection = self.relocInfosPerSection.get(sectionType)
-        if relocsInSection is not None:
-            return relocsInSection.get(vram)
-        return None
-
-    def doesSectionHasRelocs(self, sectionType: FileSectionType) -> bool:
-        return len(self.relocInfosPerSection[sectionType]) != 0
-
-
     def initGotTable(self, pltGot: int, localsTable: list[int], globalsTable: list[int]):
         self.got.initTables(pltGot, localsTable, globalsTable)
 
@@ -127,6 +113,10 @@ class Context:
                 return True
         return False
 
+    def addGlobalReloc(self, vromAddres: int, relocType: RelocType, symbol: ContextSymbol|str, addend: int=0) -> RelocationInfo:
+        reloc = RelocationInfo(relocType, symbol, addend)
+        self.globalRelocationOverrides[vromAddres] = reloc
+        return reloc
 
     def saveContextToFile(self, contextPath: Path):
         with contextPath.open("w") as f:
