@@ -6,10 +6,11 @@ from . import Utils
 from .SortedDict import SortedDict
 from .GlobalConfig import GlobalConfig, InputEndian, Compiler, Abi, ArchLevel
 from .FileSectionType import FileSectionType, FileSections_ListBasic, FileSections_ListAll
-from .ContextSymbols import SymbolSpecialType, ContextSymbol, ContextRelocInfo
+from .ContextSymbols import SymbolSpecialType, ContextSymbol
 from .SymbolsSegment import SymbolsSegment
 from .Context import Context
 from .FileSplitFormat import FileSplitFormat, FileSplitEntry
 from .ElementBase import ElementBase
 from .GlobalOffsetTable import GlobalOffsetTable
 from .OrderedEnum import OrderedEnum
+from .Relocation import RelocType, RelocationInfo, RelocationStaticReference
