@@ -5,7 +5,7 @@
 
 from __future__ import annotations
 
-from .Elf32Constants import Elf32HeaderIdentifier, Elf32ObjectFileType, Elf32HeaderFlag, Elf32SectionHeaderType, Elf32SymbolTableType, Elf32SymbolTableBinding, Elf32SymbolVisibility, Elf32SectionHeaderNumber, Elf32DynamicTable, Elf32Relocs
+from .Elf32Constants import Elf32HeaderIdentifier, Elf32ObjectFileType, Elf32HeaderFlag, Elf32SectionHeaderType, Elf32SymbolTableType, Elf32SymbolTableBinding, Elf32SymbolVisibility, Elf32SectionHeaderNumber, Elf32DynamicTable
 from .Elf32Dyns import Elf32Dyns, Elf32DynEntry
 from .Elf32GlobalOffsetTable import Elf32GlobalOffsetTable
 from .Elf32Header import Elf32Header
@@ -16,3 +16,6 @@ from .Elf32Syms import Elf32Syms, Elf32SymEntry
 from .Elf32Rels import Elf32Rels, Elf32RelEntry
 
 from .Elf32File import Elf32File
+
+# To avoid breaking backwards compatibility
+from ..common.Relocation import RelocType as Elf32Relocs
