@@ -8,7 +8,6 @@ from __future__ import annotations
 import rabbitizer
 
 from ... import common
-from ... import elf32
 
 from . import SymbolText, analysis
 
@@ -76,34 +75,58 @@ class SymbolFunction(SymbolText):
             self.instrAnalyzer.processPrevFuncCall(regsTracker, targetInstr, prevTargetInstr)
             branch += 4
 
-    def _processElfRelocSymbols(self):
-        if not self.context.doesSectionHasRelocs(self.sectionType):
-            return
+    def _runInstructionAnalyzer(self):
+        regsTracker = rabbitizer.RegistersTracker()
 
-        # Process reloc symbols
         instructionOffset = 0
-        vram = self.vram
         for instr in self.instructions:
-            relocInfo = self.context.getRelocInfo(vram, self.sectionType)
-            if relocInfo is not None:
-                if relocInfo.referencedSectionVram is not None:
-                    hiOffset = self.instrAnalyzer.lowToHiDict.get(instructionOffset)
-                    if hiOffset is not None:
-                        self.instrAnalyzer.symbolInstrOffset[hiOffset] = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
+            currentVram = self.getVramOffset(instructionOffset)
+            prevInstr = self.instructions[instructionOffset//4 - 1]
 
-                    if relocInfo.referencedSection == common.FileSectionType.Rodata:
-                        symbolVram = relocInfo.referencedSectionVram
-                        if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
-                            if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
-                                addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
-                                symbolVram += addressOffset
-                            else:
-                                symbolVram += instr.getProcessedImmediate()
-                        contextSym = self.addSymbol(symbolVram, isAutogenerated=True)
-                        if relocInfo.isStatic:
-                            contextSym._isStatic = relocInfo.isStatic
+            self.instrAnalyzer.printAnalisisDebugInfo_IterInfo(regsTracker, instr, currentVram)
 
-            vram += 4
+            if not self.isLikelyHandwritten:
+                self.isLikelyHandwritten = instr.isLikelyHandwritten()
+
+            if not common.GlobalConfig.DISASSEMBLE_UNKNOWN_INSTRUCTIONS and not instr.isImplemented():
+                # Abort analysis
+                self.hasUnimplementedIntrs = True
+                return
+
+            if not prevInstr.isBranchLikely() and not prevInstr.isUnconditionalBranch():
+                self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, prevInstr)
+
+            # look-ahead symbol finder
+            self._lookAheadSymbolFinder(instr, prevInstr, instructionOffset, regsTracker)
+
+            self.instrAnalyzer.processPrevFuncCall(regsTracker, instr, prevInstr, currentVram)
+
+            instructionOffset += 4
+
+        self.instrAnalyzer.printSymbolFinderDebugInfo_UnpairedLuis()
+
+    def _processElfRelocSymbols(self):
+        instructionOffset = 0
+        vrom = self.vromStart
+        for instr in self.instructions:
+            relocInfo = self.context.globalRelocationOverrides.get(vrom)
+            if relocInfo is not None and relocInfo.staticReference is not None:
+                hiOffset = self.instrAnalyzer.lowToHiDict.get(instructionOffset)
+                if hiOffset is not None:
+                    self.instrAnalyzer.symbolInstrOffset[hiOffset] = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
+
+                if relocInfo.staticReference.sectionType == common.FileSectionType.Rodata:
+                    symbolVram = relocInfo.staticReference.sectionVram
+                    if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
+                        if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
+                            addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
+                            symbolVram += addressOffset
+                        else:
+                            symbolVram += instr.getProcessedImmediate()
+                    contextSym = self.addSymbol(symbolVram, isAutogenerated=True)
+                    contextSym._isStatic = True
+
+            vrom += 4
             instructionOffset += 4
 
 
@@ -172,9 +195,110 @@ class SymbolFunction(SymbolText):
 
                 if loOffset in self.instrAnalyzer.indirectFunctionCallOffsets:
                     self.instrAnalyzer.indirectFunctionCallOffsets[loOffset] = symVram
-
         return
 
+    def _getRelocTypeForInstruction(self, instr: rabbitizer.Instruction, instrOffset: int, contextSym: common.ContextSymbol|None=None, gotHiLo: bool=False) -> common.RelocType:
+        if instr.canBeHi():
+            if common.GlobalConfig.PIC:
+                if contextSym is not None and gotHiLo:
+                    if contextSym.isGotGlobal and contextSym.type == common.SymbolSpecialType.function:
+                        return common.RelocType.MIPS_CALL_HI16
+                    else:
+                        return common.RelocType.MIPS_GOT_HI16
+            return common.RelocType.MIPS_HI16
+
+        if instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
+            if not common.GlobalConfig.PIC:
+                return common.RelocType.MIPS_GPREL16
+
+            if contextSym is not None:
+                if contextSym.isGotGlobal and contextSym.type == common.SymbolSpecialType.function and instrOffset in self.instrAnalyzer.indirectFunctionCallOffsets:
+                    return common.RelocType.MIPS_CALL16
+                elif contextSym.isGot:
+                    return common.RelocType.MIPS_GOT16
+
+        elif common.GlobalConfig.PIC:
+            if contextSym is not None and gotHiLo:
+                if contextSym.isGotGlobal and contextSym.type == common.SymbolSpecialType.function:
+                    return common.RelocType.MIPS_CALL_LO16
+                else:
+                    return common.RelocType.MIPS_GOT_LO16
+        return common.RelocType.MIPS_LO16
+
+    def _generateRelocsFromInstructionAnalyzer(self):
+        for instrOffset, address in self.instrAnalyzer.symbolInstrOffset.items():
+            if self.context.isAddressBanned(address):
+                continue
+
+            contextSym = self.getSymbol(address)
+
+            gotAccess = self.instrAnalyzer.gotAccessAddresses.get(instrOffset)
+            if gotAccess is not None:
+                gotEntry, inGlobalTable = self.context.got.getGotSymEntry(gotAccess)
+                if gotEntry is not None:
+                    contextSym = gotEntry.contextSym
+            if contextSym is None:
+                contextSym = self.getSymbol(address, tryPlusOffset=True)
+
+            gotHiLo = False
+            if contextSym is None and address < 0 and common.GlobalConfig.PIC and common.GlobalConfig.GP_VALUE is not None:
+                # Negative pointer may mean it is a weird GOT access
+                gotAccess = common.GlobalConfig.GP_VALUE + address
+                gotEntry, inGlobalTable = self.context.got.getGotSymEntry(gotAccess)
+                if gotEntry is not None:
+                    if gotEntry.contextSym is not None:
+                        contextSym = gotEntry.contextSym
+                    else:
+                        contextSym = self.getSymbol(gotEntry.address)
+                    gotHiLo = True
+                    address = gotEntry.address
+
+            if contextSym is None:
+                continue
+
+            if contextSym.isGotGlobal:
+                if instrOffset not in self.instrAnalyzer.gotAccessAddresses and not gotHiLo:
+                    continue
+
+            instr = self.instructions[instrOffset//4]
+
+            relocType = self._getRelocTypeForInstruction(instr, instrOffset, contextSym, gotHiLo)
+            self.relocs[instrOffset] = common.RelocationInfo(relocType, contextSym, address - contextSym.vram)
+
+        for instrOffset in self.instrAnalyzer.cploadOffsets:
+            # .cpload directive is meant to use the `_gp_disp` pseudo-symbol
+            instr = self.instructions[instrOffset//4]
+
+            relocType = self._getRelocTypeForInstruction(instr, instrOffset)
+            self.relocs[instrOffset] = common.RelocationInfo(relocType, "_gp_disp")
+
+        for instrOffset, constant in self.instrAnalyzer.constantInstrOffset.items():
+            instr = self.instructions[instrOffset//4]
+            relocType = self._getRelocTypeForInstruction(instr, instrOffset)
+
+            symbol = self.getConstant(constant)
+            if symbol is not None:
+                self.relocs[instrOffset] = common.RelocationInfo(relocType, symbol.getName())
+            elif common.GlobalConfig.SYMBOL_FINDER_FILTERED_ADDRESSES_AS_HILO:
+                self.relocs[instrOffset] = common.RelocationInfo(relocType, f"0x{constant:X}")
+            else:
+                # Pretend this pair is a constant
+                loInstr = instr
+                if instr.canBeHi():
+                    loInstr = self.instructions[self.instrAnalyzer.hiToLowDict[instrOffset] // 4]
+
+                generatedStr = self.generateHiLoConstantStr(constant, instr, loInstr)
+                if generatedStr is not None:
+                    # return generatedStr
+                    self.relocs[instrOffset] = common.RelocationInfo(common.RelocType.MIPS_NONE, generatedStr)
+
+
+        for instrOffset, targetVram in self.instrAnalyzer.funcCallInstrOffsets.items():
+            funcSym = self.getSymbol(targetVram, tryPlusOffset=False)
+            if funcSym is None:
+                continue
+            self.relocs[instrOffset] = common.RelocationInfo(common.RelocType.MIPS_26, funcSym)
+
 
     def analyze(self):
         if not common.GlobalConfig.DISASSEMBLE_UNKNOWN_INSTRUCTIONS and self.hasUnimplementedIntrs:
@@ -187,34 +311,7 @@ class SymbolFunction(SymbolText):
                 offset += 4
             return
 
-        regsTracker = rabbitizer.RegistersTracker()
-
-        instructionOffset = 0
-        for instr in self.instructions:
-            currentVram = self.getVramOffset(instructionOffset)
-            prevInstr = self.instructions[instructionOffset//4 - 1]
-
-            self.instrAnalyzer.printAnalisisDebugInfo_IterInfo(regsTracker, instr, currentVram)
-
-            if not self.isLikelyHandwritten:
-                self.isLikelyHandwritten = instr.isLikelyHandwritten()
-
-            if not common.GlobalConfig.DISASSEMBLE_UNKNOWN_INSTRUCTIONS and not instr.isImplemented():
-                # Abort analysis
-                self.hasUnimplementedIntrs = True
-                return
-
-            if not prevInstr.isBranchLikely() and not prevInstr.isUnconditionalBranch():
-                self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, prevInstr)
-
-            # look-ahead symbol finder
-            self._lookAheadSymbolFinder(instr, prevInstr, instructionOffset, regsTracker)
-
-            self.instrAnalyzer.processPrevFuncCall(regsTracker, instr, prevInstr, currentVram)
-
-            instructionOffset += 4
-
-        self.instrAnalyzer.printSymbolFinderDebugInfo_UnpairedLuis()
+        self._runInstructionAnalyzer()
 
         self._postProcessGotAccesses()
         self._processElfRelocSymbols()
@@ -227,11 +324,12 @@ class SymbolFunction(SymbolText):
             labelSym.referenceFunctions.add(self.contextSym)
 
         # Function calls
-        for targetVram in self.instrAnalyzer.funcCallInstrOffsets.values():
+        for instrOffset, targetVram in self.instrAnalyzer.funcCallInstrOffsets.items():
             funcSym = self.addFunction(targetVram, isAutogenerated=True)
             funcSym.referenceCounter += 1
             funcSym.referenceFunctions.add(self.contextSym)
 
+
         if not self.isRsp and len(self.instrAnalyzer.funcCallOutsideRangesOffsets) > 0:
             self.isLikelyHandwritten = True
 
@@ -240,11 +338,6 @@ class SymbolFunction(SymbolText):
             if self.context.isAddressBanned(symVram):
                 continue
 
-            # Check for user-defined symbol patches
-            patchedAddress = self.getLoPatch(self.getVramOffset(loOffset))
-            if patchedAddress is not None:
-                symVram = patchedAddress
-
             symAccessDict = self.instrAnalyzer.possibleSymbolTypes.get(symVram, dict())
             symAccess = None
             if len(symAccessDict) == 1:
@@ -272,6 +365,8 @@ class SymbolFunction(SymbolText):
                 contextSym.setAccessTypeIfUnset(symAccess.accessType, symAccess.unsignedMemoryAccess)
             contextSym.setFirstLoAccessIfUnset(loOffset)
 
+        self._generateRelocsFromInstructionAnalyzer()
+
         # Jump tables
         for targetVram in self.instrAnalyzer.referencedJumpTableOffsets.values():
             self.addJumpTable(targetVram, isAutogenerated=True)
@@ -376,52 +471,6 @@ class SymbolFunction(SymbolText):
             del self.instructions[first_nop:]
         return was_updated
 
-    _percentRel = {
-        elf32.Elf32Relocs.MIPS_HI16.value:      f"%hi",
-        elf32.Elf32Relocs.MIPS_LO16.value:      f"%lo",
-        elf32.Elf32Relocs.MIPS_GPREL16.value:   f"%gp_rel",
-        elf32.Elf32Relocs.MIPS_GOT16.value:     f"%got",
-        elf32.Elf32Relocs.MIPS_CALL16.value:    f"%call16",
-        elf32.Elf32Relocs.MIPS_GOT_HI16.value:  f"%got_hi",
-        elf32.Elf32Relocs.MIPS_GOT_LO16.value:  f"%got_lo",
-        elf32.Elf32Relocs.MIPS_CALL_HI16.value: f"%call_hi",
-        elf32.Elf32Relocs.MIPS_CALL_LO16.value: f"%call_lo",
-    }
-
-    def generateHiLoStr(self, instr: rabbitizer.Instruction, instrOffset: int, symName: str, symbol: common.ContextSymbol|None, relocInfo: common.ContextRelocInfo|None=None, gotHiLo: bool=False) -> str:
-        if relocInfo is not None:
-            percentStr = self._percentRel.get(relocInfo.relocType, None)
-            if percentStr is not None:
-                return f"{percentStr}({symName})"
-            common.Utils.eprint(f"generateHiLoStr: Warning: Unhandled relocType '{relocInfo.relocType}'")
-
-        if instr.canBeHi():
-            if common.GlobalConfig.PIC:
-                if symbol is not None and gotHiLo:
-                    if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function:
-                        return f"%call_hi({symName})"
-                    else:
-                        return f"%got_hi({symName})"
-            return f"%hi({symName})"
-
-        if instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
-            if common.GlobalConfig.PIC:
-                if symbol is not None:
-                    if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function and instrOffset in self.instrAnalyzer.indirectFunctionCallOffsets:
-                        return f"%call16({symName})"
-                    elif symbol.isGot:
-                        return f"%got({symName})"
-            if not common.GlobalConfig.PIC:
-                return f"%gp_rel({symName})"
-
-        if common.GlobalConfig.PIC:
-            if symbol is not None and gotHiLo:
-                if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function:
-                    return f"%call_lo({symName})"
-                else:
-                    return f"%got_lo({symName})"
-        return f"%lo({symName})"
-
     def generateHiLoConstantStr(self, constantValue: int, currentInstr: rabbitizer.Instruction, loInstr: rabbitizer.Instruction|None) -> str|None:
         if loInstr is None:
             if currentInstr.canBeHi():
@@ -436,7 +485,7 @@ class SymbolFunction(SymbolText):
                     return f"(0x{constantValue:X} & 0xFFFF)"
                 return None
             else:
-                hiHalf = constantValue > 16
+                hiHalf = constantValue >> 16
                 loHalf = constantValue & 0xFFFF
                 if loHalf < 0x8000:
                     # positive lo half
@@ -451,127 +500,51 @@ class SymbolFunction(SymbolText):
         return None
 
 
-    def getImmOverrideForInstruction(self, instr: rabbitizer.Instruction, instructionOffset: int) -> str|None:
-        # Check possible symbols using reloc information
-        relocInfo = self.context.getRelocInfo(self.vram + instructionOffset, self.sectionType)
+    def getImmOverrideForInstruction(self, instr: rabbitizer.Instruction, instrOffset: int) -> str|None:
+        if self.pointersRemoved:
+            return None
+
+        relocInfo = self.context.globalRelocationOverrides.get(self.getVromOffset(instrOffset))
         if relocInfo is not None:
-            if relocInfo.referencedSectionVram is not None:
-                relocVram = relocInfo.referencedSectionVram
+            if relocInfo.staticReference is not None:
+                relocVram = relocInfo.staticReference.sectionVram
                 addend = 0
+
                 if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
-                    if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
-                        addend = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
+                    if instrOffset in self.instrAnalyzer.symbolInstrOffset:
+                        addend = self.instrAnalyzer.symbolInstrOffset[instrOffset]
                     else:
                         addend = instr.getProcessedImmediate()
-
-                contextSym = self.getSymbol(relocVram+addend, checkUpperLimit=False)
-                if contextSym is not None:
-                    symName = contextSym.getSymbolPlusOffset(relocVram+addend)
-                    return self.generateHiLoStr(instr, instructionOffset, symName, contextSym, relocInfo=relocInfo)
-
-            auxOverride = relocInfo.getNamePlusOffset(0)
-            if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
-                if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
-                    addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
-                    auxOverride = relocInfo.getNamePlusOffset(addressOffset)
-                else:
-                    auxOverride = relocInfo.getNamePlusOffset(instr.getProcessedImmediate())
-
-                auxOverride = self.generateHiLoStr(instr, instructionOffset, auxOverride, None, relocInfo=relocInfo)
-            return auxOverride
+                labelSym = self.getSymbol(relocVram + addend, checkUpperLimit=False)
+                if labelSym is not None:
+                    relocInfo.symbol = labelSym
+                    if relocVram + addend != labelSym.vram:
+                        relocInfo.addend = (relocVram + addend) - labelSym.vram
+            return relocInfo.getNameWithReloc()
 
         if instr.isBranch() or instr.isUnconditionalBranch():
-            if not common.GlobalConfig.IGNORE_BRANCHES:
-                branchOffset = instr.getBranchOffsetGeneric()
-                targetBranchVram = self.getVramOffset(instructionOffset + branchOffset)
-                labelSymbol = self.getSymbol(targetBranchVram, tryPlusOffset=False)
-                if labelSymbol is not None:
-                    return labelSymbol.getName()
+            if common.GlobalConfig.IGNORE_BRANCHES:
+                return None
+            branchOffset = instr.getBranchOffsetGeneric()
+            targetBranchVram = self.getVramOffset(instrOffset + branchOffset)
+            labelSymbol = self.getSymbol(targetBranchVram, tryPlusOffset=False)
+            if labelSymbol is not None:
+                return labelSymbol.getName()
+            return None
 
-        elif instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
-            # .cpload directive is meant to use the `_gp_disp` pseudo-symbol
-            if instructionOffset in self.instrAnalyzer.cploadOffsets:
-                return self.generateHiLoStr(instr, instructionOffset, "_gp_disp", None)
+        relocInfo = self.relocs.get(instrOffset)
+        if relocInfo is not None:
+            return relocInfo.getNameWithReloc()
 
-            if not self.pointersRemoved and instructionOffset in self.instrAnalyzer.symbolInstrOffset:
-                address = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
-
-                if self.context.isAddressBanned(address):
-                    return None
-
-                symbol: common.ContextSymbol|None = None
-                loInstr = instr
-                instrVram = self.getVramOffset(instructionOffset)
-                if instr.canBeHi():
-                    # we need to get the address of the lo instruction to get the patch
-                    if instructionOffset in self.instrAnalyzer.hiToLowDict:
-                        loOffset = self.instrAnalyzer.hiToLowDict[instructionOffset]
-                        instrVram = self.getVramOffset(loOffset)
-                        loInstr = self.instructions[loOffset // 4]
-
-                # Check for user-defined symbol patches
-                patchedAddress = self.getLoPatch(instrVram)
-                if patchedAddress is not None:
-                    symbol = self.getSymbol(patchedAddress, tryPlusOffset=True, checkUpperLimit=False)
-                else:
-                    gotAccess = self.instrAnalyzer.gotAccessAddresses.get(instructionOffset)
-                    if gotAccess is not None:
-                        gotEntry, inGlobalTable = self.context.got.getGotSymEntry(gotAccess)
-                        if gotEntry is not None:
-                            symbol = gotEntry.contextSym
-                    if symbol is None:
-                        symbol = self.getSymbol(address, tryPlusOffset=True)
-
-                gotHiLo = False
-                if symbol is None and address < 0 and common.GlobalConfig.PIC and common.GlobalConfig.GP_VALUE is not None:
-                    # Negative pointer may mean it is a weird GOT access
-                    gotAccess = common.GlobalConfig.GP_VALUE + address
-                    gotEntry, inGlobalTable = self.context.got.getGotSymEntry(gotAccess)
-                    if gotEntry is not None:
-                        if gotEntry.contextSym is not None:
-                            symbol = gotEntry.contextSym
-                        else:
-                            symbol = self.getSymbol(gotEntry.address)
-                        gotHiLo = True
-                        address = gotEntry.address
-
-                if symbol is not None:
-                    if symbol.isGotGlobal:
-                        if instructionOffset not in self.instrAnalyzer.gotAccessAddresses and not gotHiLo:
-                            return None
-
-                    symName = symbol.getSymbolPlusOffset(address)
-                    return self.generateHiLoStr(instr, instructionOffset, symName, symbol, gotHiLo=gotHiLo)
-                return self.generateHiLoConstantStr(address, instr, loInstr)
-
-            elif instructionOffset in self.instrAnalyzer.constantInstrOffset:
-                constant = self.instrAnalyzer.constantInstrOffset[instructionOffset]
-
-                symbol = self.getConstant(constant)
-                if symbol is not None:
-                    return self.generateHiLoStr(instr, instructionOffset, symbol.getName(), symbol)
-
-                # Pretend this pair is a constant
-                loInstr = instr
-                if instr.canBeHi():
-                    loInstr = self.instructions[self.instrAnalyzer.hiToLowDict[instructionOffset] // 4]
-
-                generatedStr = self.generateHiLoConstantStr(constant, instr, loInstr)
-                if generatedStr is not None:
-                    return generatedStr
-
-                if common.GlobalConfig.SYMBOL_FINDER_FILTERED_ADDRESSES_AS_HILO:
-                    return self.generateHiLoStr(instr, instructionOffset, f"0x{constant:X}", None)
+        if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
+            if instrOffset in self.instrAnalyzer.symbolInstrOffset:
+                address = self.instrAnalyzer.symbolInstrOffset[instrOffset]
+                return self.generateHiLoConstantStr(address, instr, instr)
 
             if instr.canBeHi():
                 # Unpaired LUI
                 return self.generateHiLoConstantStr(instr.getProcessedImmediate()<<16, instr, None)
 
-        elif instr.isJumpWithAddress():
-            possibleOverride = self.getSymbol(instr.getInstrIndexAsVram(), tryPlusOffset=False)
-            if possibleOverride is not None:
-                return possibleOverride.getName()
-
         return None
 
     def getLabelForOffset(self, instructionOffset: int, migrate: bool=False) -> str:
